/*
 Stewitter.cpp - Arduino library to Post messages to Twitter with OAuth.
 Copyright (c) arms22 2010 - 2012. All right reserved.
 */
/*
 Twitter.cpp - Arduino library to Post messages to Twitter.
 Copyright (c) NeoCat 2009. All right reserved.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 */
/* Modified by 2015 Suga
 *  for a3gs library (ABIT PHS Shield)
 */

#include "Stewitter_a3gs.h"

#define STEWGATE_POST_API			"/api/post/"
#define STEWGATE_LAST_MENTION_API	"/api/last_mention/"
#define STEWGATE_HOST				"stewgate-u.appspot.com"
#define MAX_BUFFER_LENGTH			256

Stewitter::Stewitter(const char *token) : token(token)
{
    httpBody.reserve(100);
}

bool Stewitter::post(const char *msg)
{
    int r;
    char buf[MAX_BUFFER_LENGTH + 1];
    statusCode = 0;
    parseStatus = 0;
    httpBody = "";

    int length = MAX_BUFFER_LENGTH;
    char header[100], *body = buf;
    if (msg != NULL) {
        snprintf_P(body, MAX_BUFFER_LENGTH, PSTR("_t=%s&msg=%s"), token, msg);
        snprintf_P(header, sizeof(header), PSTR("Content-Length: %d\r\nContent-Type: application/x-www-form-urlencoded"), strlen(body));
        r = a3gs.httpPOST(STEWGATE_HOST, 80, STEWGATE_POST_API, header, body, buf, &length, false);
    } else {
        snprintf_P(body, MAX_BUFFER_LENGTH, PSTR("_t=%s"), token);
        snprintf_P(header, sizeof(header), PSTR("Content-Length: %d\r\nContent-Type: application/x-www-form-urlencoded"), strlen(body));
        r = a3gs.httpPOST(STEWGATE_HOST, 80, STEWGATE_LAST_MENTION_API, header, body, buf, &length, false);
        a3gs.read(&buf[length], sizeof(buf) - length);
    }

    if (r == 0) {
        parse(buf);
        return true;
    } else {
        return false;
    }
}

bool Stewitter::lastMention(void)
{
    httpBody.reserve(200);
    return post(NULL);
}

void Stewitter::parse (char *buf) {
    int i, len = 0;
 
    for (i = 0; i < strlen(buf); i ++) {
        char c = buf[i];
//        if (debug)
//            printf("%c", c);
        switch(parseStatus) {
            case 0: // skip "HTTP/1.1 "
                if (c == ' ') {
                    parseStatus++;
                }
                break;
            case 1: // parse Status Code
                if (c >= '0' && c <= '9') { 
                    statusCode *= 10;
                    statusCode += c - '0';
                } else {
                    parseStatus++;
                }
                break;
            case 2: // skip HTTP Headers
                if (c == '\x0a') {
                    parseStatus++;
                }
                break;
            case 3:
                if (c == '\x0d') {
                    parseStatus++;
                }else{
                    parseStatus = 2;
                }
                break;
            case 4:
                if (c == '\x0a') {
                    parseStatus++;
                }else{
                    parseStatus = 2;
                }
                break;
            case 5: // recv HTTP Body
                httpBody += c;
                len ++;
                break;
        }
    }
}

bool Stewitter::checkStatus(Print *debug)
{
    return false;
}

int Stewitter::wait(Print *debug)
{
	while (checkStatus(debug));
	return statusCode;
}
